#include "flydefs.cpp"
#include "keyboard.cpp"

int Game_Speed = 30;

CGround *Ground;
CList *Enemy_List;
CList *Shot_List;
CFighter* Fighter;
CScoreTable* ScoreTable;
long Tick_Count = 0;
long Up_Tick = 500;

void WaitRetrace ()
{
 asm {
  mov dx,0x3DA; }
 l1:
 asm {
  in al,dx;
  and al,8;
  jnz l1; }
 l2:
 asm {
  in al,dx;
  and al,8;
  jz l2; }
}

void cursor (char state)
{
 struct REGPACK reg;
 if (state) { reg.r_ax = 0x01 << 8; reg.r_cx = 0x0C0D; }
 else { reg.r_ax = 0x01 << 8; reg.r_cx = 0x2000; }
 intr (0x10,&reg);
}

CList::~CList ()
{
 CItem* p;
 while (head != NULL) {
  p = head;
  head = head->next;
  delete p;
 }
}

void CList::InsertLast (void *_item)
{
 CItem* x = new CItem;
 x->item = _item;
 x->next = NULL;
 CItem* p;
 for (p = head; p != NULL; p = p->next)
  if (p->next == NULL) break;
 if (p == NULL) {
  x->prev = NULL;
  head = x;
 }
 else {
  x->prev = p;
  p->next = x;
 }
 count++;
}

void CList::Delete (void* _item)
{
 CItem* p;
 for (p = head; p != NULL; p = p->next)
  if (p->item == _item) break;
 if (p == NULL) return;
 if (p->prev != NULL) p->prev->next = p->next;
 else head = p->next;
 if (p->next != NULL) p->next->prev = p->prev;
 delete p;
 count--;
 if (actual) actual--;
}

void* CList::Get (int _index)
{
 CItem* p;
 int i = 0;
 for (p = head; i < count; p = p->next, i++)
  if (i == _index) return p->item;
 return NULL;
}

void* CList::Get ()
{
 if (actual < count) {
  actual++;
  return Get (actual - 1);
 }
 return NULL;
}

void CGround::Generate ()
{
 for (Byte i = height - 1; i > 0; i--)
  for (Byte j = 0; j < width; j++)
   if (((field[i - 1][j][1] == sea_color) || (field[i - 1][j][1] == ground_color)) &&
    ((field[i][j][1] == sea_color) || (field[i][j][1] == ground_color))) {
     field[i][j][0] = field[i - 1][j][0];
     field[i][j][1] = field[i - 1][j][1];
    }
 if (!hil) if (!random (hole_probability)) {
  hole_heightl = min_hole_height + random (50);
  dxl0 = random (width / 2 - min_width + 1) / (float)(hole_heightl / 2);
  hil++;
 }
 if (!hir) if (!random (hole_probability)) {
  hole_heightr = min_hole_height + random (50);
  dxr0 = random (width / 2 - min_width + 1) / (float)(hole_heightr / 2);
  hir++;
 }
 if (hil) { dxl += dxl0; hil++; }
 if (hir) { dxr += dxr0; hir++; }
 if (dxl < 2) { dxl = 2; dxl0 = 0; }
 if (dxr < 2) { dxr = 2; dxr0 = 0; }
 if (hil == hole_heightl / 2) dxl0 = -dxl0;
 if (hir == hole_heightr / 2) dxr0 = -dxr0;
 if (hil == hole_heightl - 1) { dxl = 2; dxl0 = 0; hil = 0; }
 if (hir == hole_heightr - 1) { dxr = 2; dxr0 = 0; hir = 0; }
 Byte j = 0;
 char a = (char)dxl;
 char b = (char)dxr;
 actwidth = width - (a + b);
 actstartx = a;
 for (; j < a; j++) field[0][j][1] = ground_color;
 for (; j < width - b; j++) field[0][j][1] = sea_color;
 for (; j < width; j++) field[0][j][1] = ground_color;
}

void CGround::Paint ()
{
 WaitRetrace ();
 for (Byte i = 10; i < height - 10; i++)
  for (Byte j = 0; j < width; j++) {
   pokeb (Screen_Segment, (i - 11 + Screen_startY) * 2 * Screen_width + 2 * (j + Screen_startX - 1), field[i][j][0]);
   pokeb (Screen_Segment, (i - 11 + Screen_startY) * 2 * Screen_width + 2 * (j + Screen_startX - 1) + 1, field[i][j][1]);
  }
}

CGround::CGround ()
{
 Screen_Segment = 0xB800;
 Screen_startX = 1; Screen_startY = 4; Screen_width = 80; Screen_height = 47;
 sea_color = 0x10; ground_color = 0x60; sea_char = 32; ground_char = 32;
 width = Screen_width; height = Screen_height + 20;
 for (Byte i = 0; i < height; i++)
  for (Byte j = 0; j < width; j++)
   if ((j == 0) || (j == 1) || (j == width - 1) || (j == width - 2)) {
    field[i][j][1] = ground_color;
    field[i][j][0] = ground_char;
   }
   else {
    field[i][j][1] = sea_color;
    field[i][j][0] = sea_char;
   }
 track_length = 0;
 min_width = 5;
 min_hole_height = 50;
 hole_probability = 2;
 hil = hir = dxl0 = dxr0 = 0;
 dxl = dxr = 2;
 actwidth = width;
 actstartx = 0;
 speed = 2; spd = 0;
}

void CGround::Tick ()
{
 if (spd == speed) {
  spd = 0;
  track_length++;
  Generate ();
  down = 1;
 }
 else { spd++; down = 0; }
 Paint ();
}

char CGround::Free (Byte x, Byte y, Byte dx, Byte dy)
{
 for (Byte i = y; i < y + dy; i++)
  for (Byte j = x; j < x + dx; j++)
   if (field[i][j][1] != sea_color) return 0;
 return 1;
}

void CEnemy::Paint ()
{
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++)
   if (Body[i][j].shape) {
    ground->SetField (posX + j, posY + i, Body[i][j].shape, Body[i][j].color);
   }
}

void CEnemy::Clear ()
{
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++)
   if (Body[i][j].shape) {
    ground->SetField (posX + j, posY + i, ground->GetSeaShape (), ground->GetSeaColor ());
   }
}

char CEnemy::Create ()
{
 char is = 0;
 if (!random (enemy_probability)) {
  posX = random (ground->GetWidth () - SizeX + 1) + ground->GetStartX ();
  posY = 0;
  if (ground->Free (posX, posY, SizeX, SizeY)) is = 1;
 }
 if (is) {
  Paint ();
  enemy_list->InsertLast (this);
  return 1;
 }
 return 0;
}

void CEnemy::Destroy ()
{
 Clear ();
 enemy_list->Delete (this);
 delete this;
}

CEnemy::CEnemy (CGround *_ground, CList* _enemy_list, CFighter* _fighter)
{
 ground = _ground;
 enemy_list = _enemy_list;
 fighter = _fighter;
 state = spd = 0;
 shoot_probability = 40;
}

CEnemy::~CEnemy ()
{
 for (Byte i = 0; i < SizeY; i++) delete Body[i];
 delete Body;
}

void CEnemy::Shoot ()
{
 if (posY > 10)
  if (!random (shoot_probability)) {
   CShot *pom;
   if (dir == 1)
    pom =
     new CShot (posX + SizeX, posY + SizeY / 2, dir, ground, enemy_list, Shot_List, Fighter);
   if (dir == -1)
    pom =
     new CShot (posX - 1, posY + SizeY / 2, dir, ground, enemy_list, Shot_List, Fighter);
   if (dir == 2)
    pom =
     new CShot (posX + SizeX / 2, posY + SizeY, dir, ground, enemy_list, Shot_List, Fighter);
   if (!pom->Create ()) delete pom;
  }
}

char CEnemy::IsBody (int _x, int _y)
{
 int x = _x - posX, y = _y - posY;
 if ((x >= 0) && (x < SizeX) && (y >= 0) && (y < SizeY))
  if (Body[y][x].shape) return 1;
 return 0;
}

void CEnemy::Kill (char by_fighter)
{
 if (by_fighter) fighter->AddPoints (points);
 Destroy ();
}

CEnemy1::CEnemy1 (CGround* _ground, CList* _enemy_list, CFighter* _fighter) : CEnemy (_ground, _enemy_list, _fighter) //ISLAND
{
 SizeX = 5; SizeY = 5;
 Body = new tBody *[SizeY];
 tBody *pom;
 for (Byte k = 0; k < SizeY; k++) {
  pom = new tBody[SizeX];
  Body[k] = pom;
 }
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++) {
   Body[i][j].color = 0x50;
   Body[i][j].shape = 32;
  }
 Body[0][0].shape = Body[0][1].shape = Body[0][3].shape = Body[0][4].shape =
 Body[1][0].shape = Body[1][4].shape =
 Body[3][0].shape = Body[3][4].shape =
 Body[4][0].shape = Body[4][1].shape = Body[4][3].shape = Body[4][4].shape = 0;
}

CEnemy2::CEnemy2 (CGround* _ground, CList* _enemy_list, CFighter* _fighter) : CEnemy (_ground, _enemy_list, _fighter) //FIGHTER PLANE
{
 SizeX = 5; SizeY = 5; points = 13;
 Body = new tBody *[SizeY];
 tBody *pom;
 for (Byte k = 0; k < SizeY; k++) {
  pom = new tBody[SizeX];
  Body[k] = pom;
 }
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++) {
   Body[i][j].color = 0x00;
   Body[i][j].shape = 32;
  }
 Body[0][0].shape = Body[0][4].shape =
 Body[1][0].shape = Body[1][1].shape = Body[1][3].shape = Body[1][4].shape =
 Body[3][0].shape = Body[3][4].shape =
 Body[4][0].shape = Body[4][1].shape = Body[4][3].shape = Body[4][4].shape = 0;
 dir = 2;
}

CEnemy3::CEnemy3 (CGround* _ground, CList* _enemy_list, CFighter* _fighter) : CEnemy (_ground, _enemy_list, _fighter) //TANK
{
 SizeX = 6; SizeY = 3; points = 15;
 Body = new tBody *[SizeY];
 tBody *pom;
 for (Byte k = 0; k < SizeY; k++) {
  pom = new tBody[SizeX];
  Body[k] = pom;
 }
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++) {
   Body[i][j].color = 0x20;
   Body[i][j].shape = 32;
  }
 Body[0][5].shape =
 Body[1][0].shape =
 Body[2][5].shape = 0;
 Body[0][1].color = Body[0][3].color =
 Body[2][1].color = Body[2][3].color = 0x00;
 speed = 5; dir = 1;
}

CEnemy4::CEnemy4 (CGround* _ground, CList* _enemy_list, CFighter* _fighter) : CEnemy (_ground, _enemy_list, _fighter) //HELICOPTER
{
 SizeX = 5; SizeY = 5; points = 20;
 Body = new tBody *[SizeY];
 tBody *pom;
 for (Byte k = 0; k < SizeY; k++) {
  pom = new tBody[SizeX];
  Body[k] = pom;
 }
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++) {
   Body[i][j].color = 0x70;
   Body[i][j].shape = 32;
  }
 Body[0][0].shape = Body[0][1].shape = Body[0][3].shape = Body[0][4].shape =
 Body[1][0].shape = Body[1][1].shape = Body[1][3].shape = Body[1][4].shape =
 Body[2][4].shape =
 Body[3][0].shape = Body[3][1].shape = Body[3][3].shape = Body[3][4].shape =
 Body[4][0].shape = Body[4][1].shape = Body[4][3].shape = Body[4][4].shape = 0;
 speed = 3; dir = 1;
}

void CEnemy1::Tick ()
{
 Clear ();
 if (ground->Down ()) posY++;
 if (posY == (ground->GetHeight () - 10)) {
  Destroy ();
  return;
 }
 State ();
 Paint ();
}

void CEnemy1::State ()
{
 if (state) { Body[2][2].color = 0x40; state = 0; }
 else { Body[2][2].color = 0x50; state = 1; }
}

void CEnemy2::Tick ()
{
 Clear ();
 if (ground->Down ()) posY++;
 if (posY == (ground->GetHeight () - 10)) {
  Destroy ();
  return;
 }
 Paint ();
 Shoot ();
}

void CEnemy3::Tick ()
{
 Clear ();
 if (ground->Down ()) posY++;
 if (posY == (ground->GetHeight () - 10)) {
  Destroy ();
  return;
 }
 if (spd == speed) {
  spd = 0;
  Move ();
  State ();
 }
 else spd++;
 Paint ();
 Shoot ();
}

void CEnemy3::Move ()
{
 if (dir == 1) {
  posX++;
  if (!ground->Free (posX + SizeX - 1, posY, 1, SizeY)) {
   posX -= 2;
   dir = -dir;
   if (!ground->Free (posX, posY, 1, SizeY)) {
    posX++;
    dir = -dir;
   }
  }
 }
 else {
  posX--;
  if (!ground->Free (posX, posY, 1, SizeY)) {
   posX += 2;
   dir = -dir;
   if (!ground->Free (posX + SizeX - 1, posY, 1, SizeY)) {
    posX--;
    dir = -dir;
   }
  }
 }
 char a, b;
 if (dir == 1) { a = 0; b = 32; } else { a = 32; b = 0; }
 Body[0][5].shape =
 Body[1][0].shape =
 Body[2][5].shape = a;
 Body[0][0].shape =
 Body[1][5].shape =
 Body[2][0].shape = b;
}

void CEnemy3::State ()
{
 char a, b;
 if (state) { a = 0x20; b = 0x00; state = 0; }
 else { a = 0x00; b = 0x20; state = 1; }
 Body[0][1].color = Body[0][3].color = Body[0][5].color =
 Body[2][1].color = Body[2][3].color = Body[2][5].color = a;
 Body[0][0].color = Body[0][2].color = Body[0][4].color =
 Body[2][0].color = Body[2][2].color = Body[2][4].color = b;
}

void CEnemy4::Tick ()
{
 Clear ();
 if (ground->Down ()) posY++;
 if (posY == (ground->GetHeight () - 10)) {
  Destroy ();
  return;
 }
 if (spd == speed) {
  spd = 0;
  Move ();
 }
 else spd++;
 State ();
 Paint ();
 Shoot ();
}

void CEnemy4::Move ()
{
 if (dir == 1) {
  posX++;
  if (!ground->Free (posX + SizeX - 1, posY, 1, SizeY)) {
   posX -= 2;
   dir = -dir;
   if (!ground->Free (posX, posY, 1, SizeY)) {
    posX++;
    dir = -dir;
   }
  }
 }
 else {
  posX--;
  if (!ground->Free (posX, posY, 1, SizeY)) {
   posX += 2;
   dir = -dir;
   if (!ground->Free (posX + SizeX - 1, posY, 1, SizeY)) {
    posX--;
    dir = -dir;
   }
  }
 }
 char a, b;
 if (dir == 1) { a = 0; b = 32; } else { a = 32; b = 0; }
 Body[2][4].shape = a;
 Body[2][0].shape = b;
}

void CEnemy4::State ()
{
 char a, b, c, d;
 switch (state) {
  case 0 : { a = 0; b = 32; c = 0; d = 0; state = 1; break; }
  case 1 : { a = 0; b = 0; c = 32; d = 0; state = 2; break; }
  case 2 : { a = 32; b = 0; c = 0; d = 0; state = 3; break; }
  case 3 : { a = 0; b = 0; c = 0; d = 32; state = 0; }
 }
 if (dir == 1) Body[2][4].shape = a; else Body[2][0].shape = a;
 Body[0][2].shape =
 Body[1][2].shape =
 Body[3][2].shape =
 Body[4][2].shape = b;
 Body[0][0].shape = Body[1][1].shape = Body[3][3].shape = Body[4][4].shape = c;
 Body[0][4].shape = Body[1][3].shape = Body[3][1].shape = Body[4][0].shape = d;
}

CShot::CShot (int _posX, int _posY, char _dir, CGround* _ground, CList* _enemy_list, CList* _shot_list, CFighter* _fighter)
{
 Body.color = 0x40;
 Body.shape = 32;
 ground = _ground;
 enemy_list = _enemy_list;
 shot_list = _shot_list;
 fighter = _fighter;
 posX = _posX; posY = _posY; dir = _dir;
}

void CShot::Paint ()
{
 ground->SetField (posX, posY, Body.shape, Body.color);
}

void CShot::Clear ()
{
 ground->SetField (posX, posY, ground->GetSeaShape (), ground->GetSeaColor ());
}

char CShot::Create ()
{
 char is = 0;
 if (ground->Free (posX, posY, 1, 1)) is = 1;
 if (is) {
  Paint ();
  shot_list->InsertLast (this);
  return 1;
 }
 return 0;
}

void CShot::Destroy ()
{
 shot_list->Delete (this);
 delete this;
}

char CShot::Hit ()
{
 int save = shot_list->GetActual ();
 shot_list->SetActual (0);
 CShot* r;
 while ((r = ((CShot*)(Shot_List->Get ()))) != NULL)
  if ((r->IsBody (posX, posY)) && (r != this)) break;
 if (r != NULL) { r->Clear (); r->Destroy (); Destroy (); return 1; }
 shot_list->SetActual (save);
 enemy_list->SetActual (0);
 CEnemy* p;
 while ((p = ((CEnemy*)(Enemy_List->Get ()))) != NULL)
  if (p->IsBody (posX, posY)) break;
 if (p != NULL) { p->Kill (!dir); Destroy (); return 1; }
 if (fighter != NULL)
  if (fighter->IsBody (posX, posY)) {
   fighter->Destroy (); fighter = NULL; Fighter = NULL; Destroy (); return 1;
  }
 if (!ground->Free (posX, posY, 1, 1)) { Destroy (); return 1; }
 return 0;
}

void CShot::Tick ()
{
 if (Fighter == NULL) return;
 Clear ();
 if (ground->Down ()) posY++;
 if ((posY == (ground->GetHeight () - 10)) || (posY == 9)) {
  Destroy ();
  return;
 }
 if (dir == 0) { posY--; if (Hit ()) return; if (ground->Down ()) posY--; }
 if (dir == 1) posX++;
 if (dir == 2) posY++;
 if (dir == -1) posX--;
 if (!Hit ()) Paint ();
}

char CShot::IsBody (int _x, int _y)
{
 if ((_x == posX) && (_y == posY)) return 1;
 return 0;
}

void CFighter::Paint ()
{
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++)
   if (Body[i][j].shape) {
    ground->SetField (posX + j, posY + i, Body[i][j].shape, Body[i][j].color);
   }
}

void CFighter::Clear ()
{
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++)
   if (Body[i][j].shape) {
    ground->SetField (posX + j, posY + i, ground->GetSeaShape (), ground->GetSeaColor ());
   }
}

CFighter::CFighter (CGround* _ground, CList* _enemy_list)
{
 SizeX = 5; SizeY = 5;
 Body = new tBody *[SizeY];
 tBody *pom;
 for (Byte k = 0; k < SizeY; k++) {
  pom = new tBody[SizeX];
  Body[k] = pom;
 }
 for (Byte i = 0; i < SizeY; i++)
  for (Byte j = 0; j < SizeX; j++) {
   Body[i][j].color = 0x30;
   Body[i][j].shape = 32;
  }
 Body[4][0].shape = Body[4][4].shape =
 Body[3][0].shape = Body[3][1].shape = Body[3][3].shape = Body[3][4].shape =
 Body[1][0].shape = Body[1][4].shape =
 Body[0][0].shape = Body[0][1].shape = Body[0][3].shape = Body[0][4].shape = 0;
 ground = _ground;
 enemy_list = _enemy_list;
 tomove = toshoot = shooted = points = 0;
}

CFighter::~CFighter ()
{
 for (Byte i = 0; i < SizeY; i++) delete Body[i];
 delete Body;
}

void CFighter::Create ()
{
 posX = ground->GetSStartX () + ground->GetSWidth () / 2 - SizeX / 2;
 posY = ground->GetHeight () - 10 - SizeY;
 Paint ();
}

void CFighter::Destroy ()
{
 Clear ();
 delete this;
}

char CFighter::IsBody (int _x, int _y)
{
 int x = _x - posX, y = _y - posY;
 if ((x >= 0) && (x < SizeX) && (y >= 0) && (y < SizeY))
  if (Body[y][x].shape) return 1;
 return 0;
}

void CFighter::Tick ()
{
 Clear ();
 if (!Hit ()) {
  if (tomove) { posX += tomove; tomove = 0; if (Hit ()) return; }
  Paint ();
  if (toshoot) { toshoot = 0; DoShoot (); }
 }
}

char CFighter::Hit ()
{
 if (!ground->Free (posX, posY, SizeX, SizeY)) {
  Destroy (); Fighter = NULL; return 1;
 }
 return 0;
}

void CFighter::DoShoot ()
{
 CShot *pom;
 pom =
  new CShot (posX + SizeX / 2, posY - 1, 0, ground, enemy_list, Shot_List, Fighter);
  if (!pom->Create ()) delete pom;
}

CScoreTable::CScoreTable (CGround *_ground, CFighter* _fighter)
{
 ground = _ground; fighter = _fighter;
 gotoxy (2,2); textattr (0x0F);
 cprintf ("Skore: 0           Pocet sestrelenych: 0           Delka trasy: 0           ");
}

void CScoreTable::Update ()
{
 textattr (0x0F);
 gotoxy (9,2); cprintf ("%li", fighter->GetPoints ());
 gotoxy (41,2); cprintf ("%li", fighter->GetShootedCount ());
 gotoxy (66,2); cprintf ("%li", ground->GetTrackLength ());
}

void Enemy_Create ()
{
 CEnemy *pom;
 char t = random (4);
 switch (t) {
  case 0 : { pom = new CEnemy1 (Ground, Enemy_List, Fighter); break; }
  case 1 : { pom = new CEnemy2 (Ground, Enemy_List, Fighter); break; }
  case 2 : { pom = new CEnemy3 (Ground, Enemy_List, Fighter); break; }
  case 3 : { pom = new CEnemy4 (Ground, Enemy_List, Fighter); break; }
 }
 if (!pom->Create ()) delete pom;
}

void Timer ()
{
 Ground->Tick ();
 Enemy_List->SetActual (0);
 CEnemy* p;
 while ((p = ((CEnemy*)(Enemy_List->Get ()))) != NULL) p->Tick ();
 Shot_List->SetActual (0);
 CShot* r;
 while ((r = ((CShot*)(Shot_List->Get ()))) != NULL) r->Tick ();
 if (Fighter != NULL) Fighter->Tick ();
 if (Tick_Count > 5) Enemy_Create ();
 Tick_Count++;
 if (Tick_Count == Up_Tick) {
  Up_Tick *= 2;
  CEnemy::Up_Probability ();
 }
 ScoreTable->Update ();
 delay (500 / Game_Speed);
}

void main ()
{
 randomize ();
 textmode (64);
 textattr (0x10);
 clrscr ();
 cursor (0);
 textattr (0x1F);
 char pos = 7;
 int del = 300;
 gotoxy (pos, 24); delay (del);
 cprintf ("FFFFF  L      Y   Y        M   M  II   SSS    SSS   II   OOO   N   N"); gotoxy (pos, 25); delay (del);
 cprintf ("F      L       Y Y         MM MM  II  S      S      II  O   O  NN  N"); gotoxy (pos, 26); delay (del);
 cprintf ("FFF    L        Y          M M M  II   SSS    SSS   II  O   O  N N N"); gotoxy (pos, 27); delay (del);
 cprintf ("F      L        Y          M   M  II      S      S  II  O   O  N  NN"); gotoxy (pos, 28); delay (del);
 cprintf ("F      LLLLL    Y          M   M  II   SSS    SSS   II   OOO   N   N"); delay (del);
 pos = 21;
 gotoxy (pos, 40);
 cprintf ("Ovladani: DOLEVA       - Sipka doleva"); gotoxy (pos, 41);
 cprintf ("          DOPRAVA      - Sipka doprava"); gotoxy (pos, 42);
 cprintf ("          STRELBA      - Mezernik"); gotoxy (pos, 45);
 cprintf ("          UKONCENI HRY - Esc");
 gotoxy (20, 50);
 cprintf ("Zmackni libovolnou klavesu a uz letis !!!");
 while (!kbhit ());
 textattr (0x00);
 clrscr ();
 Ground = new CGround ();
 Enemy_List = new CList ();
 Shot_List = new CList ();
 Fighter = new CFighter (Ground, Enemy_List);
 Fighter->Create ();
 ScoreTable = new CScoreTable (Ground, Fighter);
 CKeyBoard* KeyBoard = new CKeyBoard ();
 do {
  if (KeyBoard->LeftArrow) Fighter->Move (-1);
  if (KeyBoard->RightArrow) Fighter->Move (1);
  if (KeyBoard->Space) Fighter->Shoot ();
  Timer ();
  if (Fighter == NULL) break;
 } while (!KeyBoard->Esc);
 KeyBoard->Release ();
 if (Fighter != NULL) Fighter->Destroy ();
 CEnemy* p;
 for (int i = 0; i < Enemy_List->GetCount (); i++)
  if ((p = ((CEnemy*)(Enemy_List->Get (0)))) != NULL) p->Destroy ();
 CShot* r;
 for (int j = 0; j < Shot_List->GetCount (); j++)
  if ((r = ((CShot*)(Shot_List->Get (0)))) != NULL) r->Destroy ();
 delete Enemy_List;
 delete Shot_List;
 pos = 11;
 do {
  delay (1);
  textattr (0x00);
  poke (Ground->GetSSegment (), 2 * (Ground->GetSStartX () - 1 + random (Ground->GetSWidth ())) + 2 * Ground->GetSWidth () * (Ground->GetSStartY () - 1 + random (Ground->GetSHeight ())), 0x0032);
  textattr (0x84); gotoxy (pos, 24);
  cprintf ("K   K   OOO   N   N  EEEEE   CCC        H   H  RRRR   Y   Y");  gotoxy (pos, 25);
  cprintf ("K  K   O   O  NN  N  E      C   C       H   H  R   R   Y Y "); gotoxy (pos, 26);
  cprintf ("KKK    O   0  N N N  EEE    C           HHHHH  RRRR     Y  "); gotoxy (pos, 27);
  cprintf ("K  K   O   0  N  NN  E      C   C       H   H  R  R     Y  "); gotoxy (pos, 28);
  cprintf ("K   K   OOO   N   N  EEEEE   CCC        H   H  R   R    Y  ");
 } while ((!KeyBoard->Esc) && (!KeyBoard->Space) && (!KeyBoard->Enter));
 delete KeyBoard;
 delete Ground;
 textmode (3);
 cprintf ("Zapoctova uloha do predmetu SOFTWAROVA LABORATOR, 3. semestr\r\n");
 cprintf ("Vytvoril Jan Outrata <outrataj@alpha.inf.upol.cz>");
 cursor (1);
}